(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

require "uiregister";
require "uiwinnativenode";
require "uiwindowscore";
require "uicore";
require "parameternodes";
require "winparameternodes";
require "$.winsys.__windows_gui";
require "$.gtwinnt.dgnodes";
require "$.gtwinnt.dgpcommon";
require "pnotifiers";
require "lfnhierarchy";
require "windowsgeneral";

functor uiFWinButton(Reg : UI_REGISTER
                     where type t = uiRegGUIClass.t) : UI_PLUGIN =
  struct
    structure w = WindowsGui
    val name = "Button"

    fun construct () =
          let
            val text = "Text"

            val (nameNode, _) = ParamName.create name

            (* FIXME: Notifications should really be done through the connections -
                      unless it is considered illegal for the user to connect
                      stuff to size, text, create, native etc. 
                      Maybe I should just add connection constraints?
                      Or notifications for stateinfo nodes? *)

            (* Add position and size parameter nodes to the graph *)
            val (protSizeX, protSizeY) =
                  ParamSize.addXY nameNode 100 18
            val (protPosX, protPosY) =
                  ParamPos.addXY nameNode 0 0
            val visibilityProt =
                  DGPBool.add nameNode "Visible" (true, nameNode)
            val textProt =
                  DGPString.add nameNode text ("", nameNode)
            val actionNProt =
                  DGPNotify.add
                    nameNode "OnAction" ()
            val pointerDownNProt =
                  DGPPointerButtonNotify.add
                    nameNode "OnPointerDown" ()

            fun nativeCreate () =
                    let
                      val nativeProt =
                              uiWindowsCore.createWindowsWidget
                                name "BUTTON" [w.WS_CHILD, w.BS_PUSHBUTTON]
                                true nameNode

                      (* FIXME: Use nativeProt? Or maybe nativeProt is just not needed :) *)
                      val nativeHandle = uiWinNative.getNativeHandle nameNode

                      val _ = WindowsGeneral.overrideSizePos
                                nativeHandle
                                {offsetX = 0, offsetY = 0,
                                 sizeExtraX = 0, sizeExtraY = 0}
                                {protPosX = protPosX, protPosY = protPosY,
                                 protSizeX = protSizeX, protSizeY = protSizeY}

                      val _ = WindowsGeneral.overrideText nativeHandle textProt
                      val _ = WinParamVisibility.override visibilityProt nativeHandle

                       val topNode = LFNHierarchy.firstParentOfType
                                       nameNode "Window"
                       
                       val nativeTop = uiWinNative.getNativeHandle topNode

                       val _ = w.addCommandHandler
                                 (nativeTop,
                                    uiWinNative.getNativeID nameNode,
                                    fn _ => (PPointerButtonNotify.doNotify
                                               pointerDownNProt nameNode
                                               {button = 1, point = {x = 0, y = 0}};
                                             PNotify.doNotify actionNProt nameNode ()))

                                (*w.addMessageHandler
                                  (widget, w.WM_COMMAND(*w.BN_CLICKED*)(*w.WM_LBUTTONDOWN*),
                                   fn _ => (ignore (action());
                                            SOME w.nullWord))))*)
                  in
                    ()
                  end

            val _ = ParamCreate.setup nameNode nativeCreate (fn _ => ())
          in
            nameNode
          end

    val _ = Reg.register name construct
  end
