(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

require "uiregister";
require "uiwinnativenode";
require "uiwindowscore";
require "uicore";
require "parameternodes";
require "winparameternodes";
require "$.winsys.__windows_gui";
require "$.gtwinnt.dgnodes";
require "$.gtwinnt.dgpcommon";
require "pnotifiers";
require "lfnhierarchy";
(* FIXME: Only for writing the message in the command handler... *)
require "$.basis.__int";
require "windowsgeneral";

functor uiFWinTextField(Reg : UI_REGISTER
                        where type t = uiRegGUIClass.t) : UI_PLUGIN =
  struct
    structure w = WindowsGui
    val name = "TextField"

    fun construct () =
          let
            val etext = "EditableText"

            val (nameNode, _) = ParamName.create name

            (* FIXME: Notifications should really be done through the connections -
                      unless it is considered illegal for the user to connect
                      stuff to size, text, create, native etc. 
                      Maybe I should just add connection constraints?
                      Or notifications for stateinfo nodes? *)

            
            (* Add position and size parameter nodes to the graph *)
            val (protSizeX, protSizeY) =
                  ParamSize.addXY nameNode 100 18
            val (protPosX, protPosY) =
                  ParamPos.addXY nameNode 0 0
            val visibilityProt = DGPBool.add nameNode "Visible" (true, nameNode)
            val etextProt =
                  DGPString.add nameNode etext ("", nameNode)
            val hasFocusProt =
                  DGPBool.add nameNode "HasFocus" (false, nameNode)
            val pointerDownNProt =
                  DGPPointerButtonNotify.add
                    nameNode "OnPointerDown" ()

            fun nativeCreate () =
                  let
                      val nativeProt =
                              uiWindowsCore.createWindowsWidget
                                name "EDIT" [w.ES_AUTOHSCROLL, w.WS_CHILD, w.WS_BORDER, w.WS_TABSTOP]
                                true nameNode

                      (* FIXME: Use nativeProt? Or maybe nativeProt is just not needed :) *)
                      val nativeHandle = uiWinNative.getNativeHandle nameNode
                      val nativeID = uiWinNative.getNativeID nameNode

                      val topNode =
                            LFNHierarchy.firstParentOfType
                              nameNode "Window"
                      val nativeTop = uiWinNative.getNativeHandle topNode

                      val _ = WindowsGeneral.overrideSizePos
                                nativeHandle
                                {offsetX = 0, offsetY = 0,
                                 sizeExtraX = 0, sizeExtraY = 0}
                                {protPosX = protPosX, protPosY = protPosY,
                                 protSizeX = protSizeX, protSizeY = protSizeY}

                      (* Override text *)
                      val _ = WindowsGeneral.overrideText nativeHandle etextProt

                      (* Override visibility protocol *)
                      val _ = WinParamVisibility.override visibilityProt nativeHandle

                      (* Add notifications *)
                      val _ = w.addCommandHandler
                                (nativeTop, nativeID,
                                   fn (_, cc) =>
                                      (case cc of
                                         256 => (* Officially: EN_SETFOCUS *)
                                           (PBool.write hasFocusProt true;
                                            PPointerButtonNotify.doNotify
                                               pointerDownNProt nameNode
                                               {button = 1, point = {x = 0, y = 0}})
                                       | 512 => (* Officially: EN_KILLFOCUS *)
                                           PBool.write hasFocusProt false
                                       | 768 => (* Officially: EN_CHANGE *)
                                           PString.doNotifyChanged
                                             etextProt nameNode (PString.read etextProt)
                                       | 1024 => () (* Officially: EN_UPDATE *)
                                       | 1280 => () (* Officially: 0x0500 = EN_ERRSPACE *)
                                       | 1281 => () (* Officially: 0x0501 = EN_MAXTEXT *)
                                       | 1537 => () (* Officially: 0x0601 = EN_HSCROLL *)
                                       | 1538 => () (* Officially: 0x0602 = EN_VSCROLL *)
                                       | unknownCC => print
                                           ("Notice: Something interesting has been " ^
                                            "discovered: An unknown CommandConstant code " ^
                                            "in a TextField CommandHandler! " ^
                                            "The code is: " ^ (Int.toString unknownCC))))
                  in
                    ()
                  end

            val _ = ParamCreate.setup nameNode nativeCreate (fn _ => ())
          in
            nameNode
          end

    val _ = Reg.register name construct
  end
