(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

require "uiregister";
require "uiwinnativenode";
require "uicore";
require "uiwindowscore";
require "parameternodes";
require "winparameternodes";
require "$.winsys.__windows_gui";
require "$.gtwinnt.dgnodes";
require "$.gtwinnt.dgpcommon";
require "$.gtwinnt.protocolutils";
require "$.errorhandling.exceptions";
require "pnotifiers";
require "lfnhierarchy";
require "windowsgeneral";

functor uiFWinWindow(Reg : UI_REGISTER
                     where type t = uiRegGUIClass.t) : UI_PLUGIN =
  struct
    structure w = WindowsGui
    val name = "Window"

    fun construct () =
          let
            val (nameNode, _) = ParamName.create name

            (* FIXME: Notifications should really be done through the connections -
                      unless it is considered illegal for the user to connect
                      stuff to size, text, create, native etc. 
                      Maybe I should just add connection constraints?
                      Or notifications for stateinfo nodes? *)

            (* Add position and size parameter nodes to the graph *)
            val {protPosX, protPosY, protSizeX, protSizeY,
                 getArea = getWindowArea} =
                   ParamRect.addRect nameNode
                     {x = 0, y = 0, w = 400, h = 400}
            val visibilityProt = DGPBool.add nameNode "Visible" (true, nameNode)
            val textProt =
                  DGPString.add nameNode "Text" ("", nameNode)
            val focusControlProt =
                  DGPDGNode.add nameNode "FocusControl" (NONE, nameNode)
            val pointerDownNProt =
                  DGPPointerButtonNotify.add
                    nameNode "OnPointerDown" ()
            val onCloseNProt =
                  DGPNotify.add
                    nameNode "OnClose" ()

            fun nativeCreate () =
                    let
                      val nativeProt =
                              uiWindowsCore.createWindowsWidget
                                name "Frame"
                                [(*w.WS_CHILD, only for MDI windows? *)
                                 w.WS_GROUP, w.WS_TABSTOP, (* These 3 are on Delph's TForm *)
                                 w.WS_CAPTION,
                                 w.WS_THICKFRAME, (* for resizable *)
                                 w.WS_SYSMENU,
                                 w.WS_MINIMIZEBOX, w.WS_MAXIMIZEBOX
                                 (* Notice: w.WS_POPUP for no edge.. *)]
                                false nameNode

                            (* FIXME: Use nativeProt? Or maybe nativeProt is just not needed :) *)
                            val nativeHandle = uiWinNative.getNativeHandle nameNode

                      val {write = writePosX, ...} = protPosX
                      val {write = writePosY, ...} = protPosY
                      val oldWritePosX = (!writePosX)
                      val oldWritePosY = (!writePosY)

                      val _ = WindowsGeneral.overrideSizePos
                                nativeHandle
                                {offsetX = 4, offsetY = 23,
                                 sizeExtraX = 8, sizeExtraY = 24}
                                {protPosX = protPosX, protPosY = protPosY,
                                 protSizeX = protSizeX, protSizeY = protSizeY}

                      val _ = WindowsGeneral.overrideText nativeHandle textProt

                      val _ = WinParamVisibility.override visibilityProt nativeHandle
                            
                            (* FIXME: Is this necessary? *)
                            val _ = w.showWindow (nativeHandle, w.SW_SHOWDEFAULT)

                            (* FIXME: Event handler on close *)
                            val _ = w.addMessageHandler
                                      (nativeHandle, w.WM_CLOSE,
                                       fn _ => (PNotify.doNotify onCloseNProt nameNode ();
                                                w.destroyWindow nativeHandle;
                                                SOME w.nullWord))
                            
                            val _ = w.addMessageHandler
                                      (nativeHandle, w.WM_LBUTTONDOWN(*w.WM_LBUTTONDBLCLK*),
                                       fn _ => (PPointerButtonNotify.doNotify
                                                  pointerDownNProt nameNode
                                                  {button = 1, point = {x = 0, y = 0}};
                                                  SOME w.nullWord))

                            (*
                            val _ = w.addCommandHandler
                                 (nativeHandle,
                                    uiWinNative.getNativeID nameNode,
                                    fn _ => PPointerButtonNotify.doNotify
                                              pointerDownNProt nameNode
                                              {button = 1, point = {x = 0, y = 0}})*)

                            val _ = w.addMessageHandler
                               (nativeHandle, w.WM_MOVE,
                                fn (w.WPARAM(wparam), w.LPARAM(lparam)) =>
                                  (oldWritePosX (w.loword lparam);
                                   oldWritePosY (w.hiword lparam);
                                   (* FIXME: Resize? *)
                                   SOME w.nullWord))
                  in
                    ()
                  end

            val _ = ParamCreate.setup nameNode nativeCreate (fn _ => ())
          in
            nameNode
          end

    val _ = Reg.register name construct
  end
