(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

require "$.gtwinnt.dgnodes";
require "$.gtwinnt.dgprotocols";
require "$.typeneutral.classes";
require "$.winsys.__windows_gui";

structure uiPWinNative =
  struct
    (* NOTICE: This is not implemented with GN.PMethod like other protocols... *)
    type t = {readHandle : unit -> WindowsGui.hwnd,
              readID : unit -> WindowsGui.word}

    structure P =
      FClass(type newType = t
             val className = "Windows native")

    fun readHandle ({readHandle, ...} : t) = readHandle ()
    fun readID ({readID, ...} : t) = readID ()
  end

functor uiFWinNative() =
  struct
    fun create winHandle id =
          let
            val protocol = uiPWinNative.P.create
                             {readHandle = (fn () => winHandle),
                              readID = (fn () => id)}
          in
            DGNodes.create "Windows native" protocol
          end

    fun getNativeProtocol graphNode =
          let
            val nativeNode = DGNodes.singleConnectedToType graphNode "Windows native"
          in
            DGNodes.protocol uiPWinNative.P.c nativeNode
          end

    fun getNativeHandle graphNode =
          uiPWinNative.readHandle (getNativeProtocol graphNode)

    fun getNativeID graphNode =
          uiPWinNative.readID (getNativeProtocol graphNode)
  end

structure uiWinNative = uiFWinNative()
